/*
This is part of Screenbar, which is based in part on the Litestep
shell source code.

Copyright (C) 2008-2009 Jim Babcock
Copyright (C) 1997-2009 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#pragma once

struct LoadedModuleInstance;
typedef int (*ModuleInitExFunc)(HWND, HINSTANCE, LPCSTR);
typedef int (*ModuleQuitFunc)(HINSTANCE);

class ChildModuleElement
	:public RectLayoutElement
{
public:
	ChildModuleElement(string prefix);
	~ChildModuleElement();
	
	bool changed(const LayoutCacheNode *node);
	ElementSize getLength(const LayoutCacheNode *node);
	Rect getRect(const LayoutCacheNode *node);
	void arrangeChildren(LayoutCacheNode *container, vector<LayoutCacheNode*> &children);
	
	void resizedEvent(string newWidth, string newHeight);
	
protected:
	FARPROC getModuleFunc(HINSTANCE dllInstance, const char *name);
	void updateVars(bool doBangs, Rect boundingRect, LoadedModuleInstance *instance);
	
	LoadedModuleInstance *getInstance(const LayoutCacheNode *node);
	LoadedModuleInstance *createInstance(LayoutCacheNode *node);
	void destroyInstance(LoadedModuleInstance *instance);
	
	map<ScreenbarWindow*, LoadedModuleInstance*> instances;
	
	string dllName;
	bool singleton;
	string modulePrefix;
	string moveCommand;
	string resizeCommand;
	string varX;
	string varY;
	vector<string> varWidth;
	vector<string> varHeight;
	string resizedEventCommand;
};

struct LoadedModuleInstance
{
	ChildModuleElement *element;
	HINSTANCE dllInstance;
	HWND containerWindow;
	
	ModuleInitExFunc initFunc;
	ModuleQuitFunc quitFunc;
	
	string width, height;
	
	int lastX, lastY;
	int lastWidth, lastHeight;
	bool varsChanged;
};

class ModuleLoadException
	{};
